#include "explore_param.h"

void initGD44(struct csr_matrix *G, struct csr_matrix *D, int N, int M,
              double alphaG, double betaG, double gammaG,
              double alphaD, double betaD, double gammaD)
{
    double *Ga, *Gb, *Da;
    double g[6];
    int maxnz;
    MKL_INT job[5];
    MKL_INT info;
    MKL_INT m, n;

    int i, j;

    maxnz = 6 * 4 * 2 + 4 * (MAX(N,M) - 8);

    Ga = (double *)mkl_calloc( (N+1) * (N+2), sizeof( double ), 64 );
    Gb = (double *)mkl_calloc( (N+1) * (N), sizeof( double ), 64 );
    Da = (double *)mkl_calloc(  M    * (M+1), sizeof( double ), 64 );

    G->a  = (double *)  mkl_calloc( maxnz, sizeof( double ), 64);
    G->ja = (MKL_INT *) mkl_calloc( maxnz, sizeof( MKL_INT), 64);
    G->ia = (MKL_INT *) mkl_calloc( N+1,   sizeof( MKL_INT), 64);
    D->a  = (double *)  mkl_calloc( maxnz, sizeof( double ), 64); 
    D->ja = (MKL_INT *) mkl_calloc( maxnz, sizeof( MKL_INT), 64);
    D->ia = (MKL_INT *) mkl_calloc( M,     sizeof( MKL_INT), 64);

    job[0] = 0;
    job[1] = 0;
    job[2] = 0;
    job[3] = 2;
    job[4] = maxnz;
    job[5] = 1;

    g[0] =  0.0;
    g[1] =  1./24.;
    g[2] = -27./24.;
    g[3] =  27./24.;
    g[4] = -1./24.;
    g[5] =  0.0;

    Ga[CELLG(1,1)] =(-124832./42735.) + (16512./1295.)*alphaG+(18816./2035.)*betaG +(13696./1295.)*gammaG;
    Ga[CELLG(1,2)]= (10789./3256.) - (1161./37.)*alphaG -(9261./407.)*betaG - (963./37.)*gammaG;
    Ga[CELLG(1,3)]= (-421./9768.) + (1548./37.)*alphaG +(12348./407.)*betaG + (1284./37.)*gammaG;
    Ga[CELLG(1,4)]= (-12189./16280.) - (6966./185.)*alphaG -(55566./2035.)*betaG-(5778./185.)*gammaG;
    Ga[CELLG(1,5)]= (11789./22792.) + (4644./259.)*alphaG + (5292./407.)*betaG + (3852./259.)*gammaG;
    Ga[CELLG(1,6)]= (-48./407.) - (129./37.)*alphaG -(1029./407.)*betaG - (107./37.)*gammaG;

    Ga[CELLG(2,1)]=16./105. - (128./35.)*alphaG;
    Ga[CELLG(2,2)]=-31./24. + 9.*alphaG;
    Ga[CELLG(2,3)]=29./24. - 12.*alphaG;
    Ga[CELLG(2,4)]=-3./40. + (54./5.)*alphaG;
    Ga[CELLG(2,5)]=1./168. - (36./7.)*alphaG;
    Ga[CELLG(2,6)]=alphaG;

    Ga[CELLG(3,1)]=-betaG*128./35.;
    Ga[CELLG(3,2)]=1./24. + 9.*betaG;
    Ga[CELLG(3,3)]=-27./24. - 12.*betaG;
    Ga[CELLG(3,4)]=27./24. + (54./5.)*betaG;
    Ga[CELLG(3,5)]=-1./24. - (36./7.)*betaG;
    Ga[CELLG(3,6)]=betaG;

    Ga[CELLG(4,1)]=-16./105. - gammaG*128./35.;
    Ga[CELLG(4,2)]=3./8. + 9.*gammaG;
    Ga[CELLG(4,3)]=-11./24. - 12.*gammaG;
    Ga[CELLG(4,4)]=-27./40. + (54./5.)*gammaG;
    Ga[CELLG(4,5)]=51./56. - (36./7.)*gammaG;
    Ga[CELLG(4,6)]=gammaG;

    for(i=1;i <= N+2; i++)
    {
      Ga[CELLG(N+1,i)] = -Ga[CELLG(1,N+3-i)];
      Ga[CELLG(N,i)]   = -Ga[CELLG(2,N+3-i)]; 
      Ga[CELLG(N-1,i)] = -Ga[CELLG(3,N+3-i)];
      Ga[CELLG(N-2,i)] = -Ga[CELLG(4,N+3-i)];
    }

    for(i=5;i <= N-3; i++)
      for(j = i-2; j <= i+3; j++)
        Ga[CELLG(i,j)] = g[j-i+2];

    for(i = 0; i < N+1; i++)
      for(j = 0; j < N; j++)
      {
        Gb[i*N+j] = Ga[i*(N+2) + j+1];
      }

    Da[CELLD(1,1)]=-6851./7788. +39./59.*alphaD +675./649.*betaD +551./649.*gammaD; 
    Da[CELLD(1,2)]=8153./15576. -195./59.*alphaD -3375./649.*betaD -2755./649.*gammaD; 
    Da[CELLD(1,3)]=3867./5192. +390./59.*alphaD +6750./649.*betaD +5510./649.*gammaD; 
    Da[CELLD(1,4)]=-9005./15576. -390./59.*alphaD - 6750./649.*betaD - 5510./649.*gammaD; 
    Da[CELLD(1,5)]=3529./15576. +195./59.*alphaD +3375./649.*betaD +2755./649.*gammaD; 
    Da[CELLD(1,6)]=-24./649. -39./59.*alphaD - 675./649.*betaD - 551./649.*gammaD;

    Da[CELLD(2,1)]=1./24. -alphaD; 
    Da[CELLD(2,2)]=-27./24. + 5.*alphaD; 
    Da[CELLD(2,3)]=27./24. -10.*alphaD; 
    Da[CELLD(2,4)]=-1./24. +10.*alphaD;
    Da[CELLD(2,5)]=-5.*alphaD; 
    Da[CELLD(2,6)]=alphaD;

    Da[CELLD(3,1)]=-betaD; 
    Da[CELLD(3,2)]=1./24. + 5.*betaD; 
    Da[CELLD(3,3)]=-27./24. -10.*betaD; 
    Da[CELLD(3,4)]=27./24. +10.*betaD; 
    Da[CELLD(3,5)]=-1./24. -5.*betaD;
    Da[CELLD(3,6)]=betaD;

    Da[CELLD(4,1)]=-1./24. -gammaD; 
    Da[CELLD(4,2)]=5./24. + 5.*gammaD; 
    Da[CELLD(4,3)]=-3./8. -10.*gammaD; 
    Da[CELLD(4,4)]=-17./24. +10.*gammaD;
    Da[CELLD(4,5)]=11./12. -5.*gammaD; 
    Da[CELLD(4,6)]=gammaD;

    for(i=1;i <= M+1; i++)
    {
      Da[CELLD(M,i)]   = -Da[CELLD(1,M+2-i)];
      Da[CELLD(M-1,i)] = -Da[CELLD(2,M+2-i)];
      Da[CELLD(M-2,i)] = -Da[CELLD(3,M+2-i)];     
      Da[CELLD(M-3,i)] = -Da[CELLD(4,M+2-i)];     
    }

    for(i=5;i <= M-4; i++)
      for(j = i-2; j <= i+3; j++)
        Da[CELLD(i,j)] = g[j-i+2];

    m = N + 1;
    n = N; 

    mkl_ddnscsr(job , &m , &n ,  Gb , &n , G->a, 
                G->ja , G->ia , &info);

    m = M;
    n = M+1; 

    mkl_ddnscsr(job , &m , &n ,  Da , &n , D->a, 
                D->ja , D->ia , &info);

   mkl_free(Ga); 
   mkl_free(Gb);
   mkl_free(Da);
}

