#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <complex.h>
#include <fftw3.h>

#include "explore_param.h"

void hilbert(int N, double *signal, double complex *analyt_sig);

// Compute the phase and envelope misfits using hilbert transforms  
void pe_misfit(int N, double *ref, double *sig, double *pm, double *em)
{
  double  maxref, d;
  double complex *href, *hsig;

  int i;

  href = (double complex*) calloc(N, sizeof(double complex));
  hsig = (double complex*) calloc(N, sizeof(double complex));

  // Copy signals (this could have been a memcpy)
  for(i = 0; i < N; i++)
  {
    href[i] = ref[i];
    hsig[i] = sig[i];
  }

  // Apply Hilbert transforms to both the signal and the reference
  hilbert(N,ref,href);
  hilbert(N,sig,hsig);

  *pm = 0.0;
  *em = 0.0;
   d  = 0.0;
 
  // Compute the misfits
  for(i = 0; i < N; i++)
  {
    double complex CREF = href[i]; 
    double complex CSIG = hsig[i];
    double aref = cabs(CREF);
    double asig = cabs(CSIG);
    double np   = aref * carg(CREF/CSIG);
    //tem[i] = aref - asig;
    //tpm[i] = aref * np / PI;
    //maxref = (maxref < aref) ? maxref = aref : maxref;
   *pm += np * np;
    d  += aref*aref;
   *em += (aref-asig) * (aref-asig);
  }

  *pm = sqrt(*pm) / PI / sqrt(d);
  *em = sqrt(*em) / sqrt(d);

  //maxref = 1.0 / maxref;
}

// Hilbert transforms using FFTW
void hilbert(int N, double *signal, double complex *analyt_sig)
{
  fftw_complex *in,*out, *aux;

  int i;
  
  in  = (fftw_complex*) fftw_malloc(N * sizeof(fftw_complex)); // input
  aux = (fftw_complex*) fftw_malloc(N * sizeof(fftw_complex)); // input
  out = (fftw_complex*) fftw_malloc(N * sizeof(fftw_complex)); // output

  for(i = 0; i < N; i++)
  {
    in[i]   = signal[i];
  }

  fftw_plan pf = fftw_plan_dft_1d(N, in, aux, FFTW_FORWARD, FFTW_ESTIMATE);
  fftw_execute(pf);
  fftw_destroy_plan(pf);

  if(N % 2 == 0)
  {
    for(i = 0; i < N; i++)
    {   
      if(i != 0 && i != N/2)
      {
        if( i < N/2 )
          aux[i]   *= 2.0;

        if( i > N/2 )
          aux[i]   = 0.0;
      }
    }
  }
  else
  {
    for(i= 0; i < N; i++)
    {   
      if(i != 0)
      {
        if( i < N/2 )
          aux[i]   *= 2.0;

        if( i > N/2 )
          aux[i]   = 0.0;
      }
    }
  }

  fftw_plan pb = fftw_plan_dft_1d(N, aux, out, FFTW_BACKWARD, FFTW_ESTIMATE);

  fftw_execute(pb);
  fftw_destroy_plan(pb);

  memcpy(analyt_sig, out, N*sizeof(fftw_complex));

  fftw_free(in);
  fftw_free(aux);
  fftw_free(out); 

  for(i = 0; i < N; i++)
    analyt_sig[i] = creal(analyt_sig[i]) / N + (cimag(analyt_sig[i]) / N) * I;
}
